# 封装基本方法，包括元素查找，元素定位
from selenium import webdriver
from selenium.common.exceptions import NoSuchElementException, TimeoutException
from selenium.webdriver.remote.webelement import WebElement
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.support import expected_conditions as ec


class Base(object):

    def __init__(self, driver: webdriver, timeout=10):
        self.driver = driver
        self.timeout = timeout

    def find_element(self, by: str, location: str) -> WebElement:
        """
        find element by location
        :param by: xpath, name, id ...
        :param location:  //*[name = 'add']
        :return: WebElement
        :Usage:
            locator = ('xpath', //*[@id='location'])
            element = driver.find_element(*locator)
        """
        by = by.lower()
        wait = WebDriverWait(self.driver, self.timeout)
        element = wait.until(lambda driver: driver.find_element(by, location))
        return element

    def find_elements(self, by: str, location: str) -> list:
        """
        find elements by location
        :param by: xpath, name, id ...
        :param location: //*[name = 'add']
        :return:
        """
        by = by.lower()
        wait = WebDriverWait(self.driver, self.timeout)
        elements = wait.until(lambda driver: driver.find_elements(by, location))
        return elements

    def is_element_exist(self, by: str, location: str)->bool:
        """
        if element is exist return true else return false
         :param by: xpath, name, id ...
        :param location:  //*[name = 'add']
        :return: bool
        """
        self.timeout = 10
        by.lower()
        try:
            wait = WebDriverWait(self.driver, self.timeout)
            wait.until(ec.visibility_of_element_located((by, location)))
        except (NoSuchElementException, TimeoutException):
            return False
        return True

    def is_clickable(self, by: str, location: str):
        """
        if element is klickable return true else return false
        :param by:
        :param location:
        :return: bool
        """
        by.lower()
        wait = WebDriverWait(self.driver, self.timeout)
        element = wait.until(ec.element_to_be_clickable((by, location)))
        return element

    def click(self, by: str, location: str):
        """
        单个元素进行点击，
        :param by:
        :param location:
        :return:
        """
        element = self.is_clickable(by, location)
        element.click()

    # 当加上@property属性后，可以直接按照访问属性的方式去访问方法，即去掉方法名后面的（）
    # 页面中调用时，该被调用的方法（base文件内）若加上@property属性，则self.get_alert_text
    # @property
    def is_alert_switch_to_it(self):
        """
        当为alert弹框时，采用这个方法 但是项目中是div弹框，不是alert
        :return:
        """
        self.timeout = 10
        wait = WebDriverWait(self.driver, self.timeout)
        alert = wait.until(ec.alert_is_present())
        return alert




"""
读取TXT文件中的要运行的测试模块，并按顺序执行
"""
import os
import unittest


class RunSet:
    # 初始化 先有实例，才能初始化
    def __init__(self, root_path):
        """
        加载测试用例初始化
        :param root_path: 项目的主路径
        """
        self.root_path = root_path
        # 项目主路径
        self.ROOT_PATH = self.root_path
        # 测试用例方法路径
        self.CASE_PATH = os.path.join(self.ROOT_PATH, 'testCase')

    def set_case_list(self):
        """读取TXT文件中的运行模块，返回列表"""
        case_list = []
        case_list_path = os.path.join(self.ROOT_PATH, "caseList.txt")
        fb = open(case_list_path, encoding='UTF-8')
        for case in fb.readlines():
            case_name = str(case)
            if case_name != "" and not case_name.startswith("#"):
                case_list.append(case_name.replace("\n", ""))
        fb.close()
        return case_list

    def set_suite(self):
        """循环测试用例的列表，用名字匹配testCase下的py文件，加载测试用例"""
        suite_list = unittest.TestSuite()
        suite_module = []
        case_list = self.set_case_list()

        for case in case_list:
            case_name = str(case)
            # TestLoader类中提供的discover（）方法可以自动识别测试用例,
            # pattern=：表示用例文件名的匹配原则。此处匹配以“ ”开头的.py 类型的文件
            discover = unittest.defaultTestLoader.discover(self.CASE_PATH, pattern=case_name + '.py', top_level_dir=None)
            if len(discover._tests) != 0:
                suite_module.append(discover)
        if len(suite_module) > 0:
            for case in suite_module:
                suite_list.addTest(case)  # 通过TestSuite类的addTest（）方法按照一定的顺序来加载
        else:
            return None
        return suite_list



"""
自动化测试，Windows系统文件，打开弹窗，文件上传类

"""
import win32gui
import win32con


class Upload:

    def __init__(self, file_path):
        """
        文件上传初始化
        :param file_path: 文件的路径，拼接到文件名称
        """
        self.file_path = file_path

    # 1\找到输入框和打开按钮 元素；2、输入地址，点击打开。
    # edit - combox - comboBoxEx32 - #32770
    # 前提 ：windows上传窗口已经出现。sleep1-2秒等待弹出的出现。
    def upload(self, browser_type="chrome"):
        if browser_type == "chrome":
            title = "打开"
        else:
            title = ""

        # 找元素
        # 一级窗口"#32770","打开"
        dialog = win32gui.FindWindow("#32770", title)
        ComboBoxEx32 = win32gui.FindWindowEx(dialog, 0, "ComboBoxEx32", None)  # 二级
        comboBox = win32gui.FindWindowEx(ComboBoxEx32, 0, "ComboBox", None)  # 三级
        # 编辑按钮
        edit = win32gui.FindWindowEx(comboBox, 0, 'Edit', None)  # 四级
        # 打开按钮
        button = win32gui.FindWindowEx(dialog, 0, 'Button', "打开(&O)")  # 二级

        # 往编辑当中，输入文件路径 。
        win32gui.SendMessage(edit, win32con.WM_SETTEXT, None, self.file_path)  # 发送文件路径
        win32gui.SendMessage(dialog, win32con.WM_COMMAND, 1, button)  # 点击打开按钮


# if __name__ == '__main__':
#
#     file_path = r'C:\Users\ithinkdt\Pictures\ThinDT-S.jpg'
#     Upload(file_path).upload()




# !usr/bin/env python
# _*_ coding: utf-8 _*_

from setuptools import setup
from setuptools import find_packages


setup(
    name='ithinkdt-auto-test',  # 项目代码所在目录，也是pip 要上传的项目名称
    version='0.2.0',  # 工具的版本号
    description='auto test libs',
    long_description='python libs of auto test include ddt, base, run_set, upload',
    license='MIT',
    author='huiLi',
    author_email='lihui@ithinkdt.com',
    packages=find_packages(where='./ithinkdt-auto-test'),  # 查找包的路径
    package_dir={'': 'ithinkdt-auto-test'},      # 包的root 路径映射到的实际路径
    install_requires=['selenium', 'pywin32'],
    platforms='any'
)
